﻿using CelloSaaS.AuthorizationServer.Models;
using CelloSaaS.AuthServer.Core;
using CelloSaaS.AuthServer.Core.Ancilliary;
using CelloSaaS.AuthServer.Core.Services.CacheServices;
using CelloSaaS.AuthServer.Core.Services.Clients;
using System;
using System.Net;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http;

namespace CelloSaaS.AuthorizationServer.Controllers
{
    [RoutePrefix(Routes.CacheRoutePrefix)]
    public class CacheController : ApiController
    {
        [HttpGet]
        public IHttpActionResult Clear()
        {
            FlushObjectCache();
            FlushResourcesCache();
            FlushAppSettings();
            ClearUserCache();
            return Ok<string>(Localization.CacheResource.msg_CacheClearSuccess);
        }

        [Route(Routes.ClearUserCacheRoute)]
        [HttpGet]
        public IHttpActionResult ClearUserCache()
        {
            try
            {
                CelloUserCacheService.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult();
            }
            return Ok<string>(Localization.CacheResource.msg_UserCacheClearSuccess);
        }

        [Route(Routes.ClearObjectCacheRoute)]
        [HttpGet]
        public IHttpActionResult FlushObjectCache()
        {
            try
            {
                AuthenticationProvidersCacheService.Flush();
                AuthenticationTypesCacheService.Flush();
                ClientCacheService.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult();
            }
            return Ok<string>(Localization.CacheResource.msg_ObjCacheClearSuccess);
        }

        [Route(Routes.ClearResourceCacheRoute)]
        [HttpGet]
        public IHttpActionResult FlushResourcesCache()
        {
            try
            {
                StaticResourceHelpers.Flush();
                StaticResourceHelpers.Init();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult();
            }
            return Ok<string>(Localization.CacheResource.msg_RsrcCacheClearSuccess);
        }

        [Route(Routes.ClearApplicationSettings)]
        [HttpGet]
        public IHttpActionResult FlushAppSettings()
        {
            try
            {
                AppSettingHelpers.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult();
            }
            return Ok<string>(Localization.CacheResource.msg_AppCacheClearSuccess);
        }
    }

    public class ErrorResult : IHttpActionResult
    {
        private readonly string _errorMessage;

        public ErrorResult()
        {
        }

        public ErrorResult(string errorMessage)
        {
            _errorMessage = errorMessage;
        }

        public async System.Threading.Tasks.Task<HttpResponseMessage> ExecuteAsync(System.Threading.CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(_errorMessage))
            {
                return new HttpResponseMessage(HttpStatusCode.InternalServerError)
                {
                    ReasonPhrase = Localization.CacheResource.ex_CacheClearFail,
                    Content = new StringContent(_errorMessage)
                };
            }

            return new HttpResponseMessage(HttpStatusCode.InternalServerError)
            {
                ReasonPhrase = Localization.CacheResource.ex_CacheClearFail,
                Content = new StringContent(Localization.CacheResource.ex_CacheClearFail)
            };
        }
    }
}
